package com.shixing.sxedit;

import com.shixing.sxedit.types.SXEffectType;

/**
 * 内置类型
 * */

public class SXGenericEffect extends SXEffect {


    /**
     * 颜色调节特效属性
     */
    public enum ColorAdjustmentAttrs {
        /**
         * 亮度 值类型: float, 值范围: [0, 1]
         */
        Brightness(1, "brightness"),
        /**
         * 对比度 值类型: float, 值范围: [-1, 1]
         */
        Contrast(1 << 1, "contrast"),
        /**
         * 饱和度 值类型: float, 值范围: [-1, 1]
         */
        Saturation(1 << 2, "saturation"),
        /**
         * 锐化 值类型: float, 值范围: [-1, 1]
         */
        Sharpen(1 << 3, "sharpen"),
        /**
         * 高光 值类型: float, 值范围: [0, 1]
         */
        Highlight(1 << 4, "highlight"),
        /**
         * 阴影 值类型: float, 值范围: [0, 1]
         */
        Shadow(1 << 5, "shadow"),
        /**
         * 曝光度 值类型: float, 值范围: [-1, 1]
         */
        Exposure(1 << 6, "exposure"),
        /**
         * 色相 值类型: float, 值范围: [0, 1]
         */
        Hue(1 << 7, "hue"),
        /**
         * 色温 值类型: float, 值范围: [-1, 1]
         * */
        Cct(1 << 8, "cct");

        public final int nValue;
        public final String name;

        ColorAdjustmentAttrs(int value, String name) {
            this.nValue = value;
            this.name = name;
        }
    }


    /**
     * 抠图特效属性
     * */
    public enum ChromaKeyAttrs {
        /**
         * 颜色, 值类型: String, 值范围: 十六进制颜色字符串: #00FF00
         * */
        COLOR("color"),
        /**
         * 相似度 值类型: float, 值范围: [0, 1]
         * */
        SIMILARITY("similarity"),
        /**
         * 平滑度 值类型: float, 值范围: [0, 1]
         * */
        SMOOTHNESS("smoothness"),
        /**
         * 溢色消除 值类型: float, 值范围: [0, 1]
         * */
        SPILL_REDUCE("spill_reduce"),
        /**
         * 边缘缩进 值类型: int, 值范围: [0, 100]
         * */
        EDGE_THIN("edge_thin"),
        /**
         * 边缘羽化 值类型: int, 值范围: [0, 50]
         * */
        EDGE_FEATHER("edge_feather");

        private final String name;
        ChromaKeyAttrs(String value) {
            name = value;
        }
    }

    /**
     * 美颜特效属性
     * */
    public enum FaceBeautyAttrs {
        /**
         * 磨皮强度  值类型: float, 值范围: [0, 30]
         * */
        BLUR("blur"),
        /**
         * 磨皮范围  值类型: float, 值范围: [0, 100]
         * */
        SKIN_RANGE("skin_range"),
        /**
         * 皮肤美白  值类型: float, 值范围: [0, 100]
         * */
        WHITEN("whiten"),
        /**
         * 皮肤红润  值类型: float, 值范围: [0, 100]
         * */
        REDDEN("redden"),
        /**
         * 皮肤粉嫩  值类型: float, 值范围: [0, 100]
         * */
        PINKING("pinking"),
        /**
         * 皮肤颜色  值类型: float, 值范围: [0, 1]
         * */
        SKIN_HUE("skin_hue");

        public final String name;

        FaceBeautyAttrs(String name) {
            this.name = name;
        }
    }

    /**
     * 轨道遮罩属性
     * */
    public enum TrackMatteAttrs {
        /**
         * 遮罩文件路径, 值类型: String, 值范围: 本地文件路径
         * */
        PATH("path"),
        /**
         * 遮罩类型, 值类型: int, 范围 : 0:亮度，1:反亮度，2:alpha，3:反alpha
         * */
        TYPE("type");
        public final String name;

        TrackMatteAttrs(String name) {
            this.name = name;
        }
    }

    /**
     * 快速模糊属性
     * */
    public enum FastBlurAttrs {
        /**
         * 模糊力度, 值类型: int, 范围: [0,8], 默认值: 2,
         * 0与1效果相同
         * */
        STRENGTH("strength"),
        /**
         * 模糊半径, 值类型: float, 范围: [0, 60], 默认值: 8
         * 尽量增大模糊力度,减小模糊半径
         * */
        RADIUS("radius");

        public final  String name;
        FastBlurAttrs(String name) {this.name = name;}
    }

    public SXGenericEffect(long nativeEffect, long nativeManager, String trackID) {
        super(nativeEffect, nativeManager, trackID);
    }

    @Override
    public SXEffectType getType() {
        return SXEffectType.GenericEffect;
    }

    /**
     * 设置 float 类型属性的值
     *
     * @param name, 属性值
     * @param value 值
     * @return boolean
     * */
    public boolean setAttributeFloat(String name, float value) {
        if (getNativeEffect() > 0) {
            return nGenericEffectSetAttributeFloat(mNativeEffect, name, value);
        }
        return false;
    }
    /**
     * 设置 float 类型属性的值
     *
     * @param name, 属性值
     * @param value 值
     * @return boolean
     * */
    public boolean setAttributeInt(String name, int value) {
        if (getNativeEffect() > 0) {
            return nGenericEffectSetAttributeInt(mNativeEffect, name, value);
        }
        return false;
    }

    /**
     * 设置 String 类型属性的值
     *
     * @param name, 属性值
     * @param value 值
     * @return boolean
     * */
    public boolean setAttributeString(String name, String value) {
        if (getNativeEffect() > 0) {
            return nGenericEffectSetAttributeString(mNativeEffect, name, value);
        }
        return false;
    }
    /**
     * 设置 Color 类型属性的值
     *
     * @param name, 属性值
     * @param value 值
     * @return boolean
     * */
    public boolean setAttributeColor(String name, String value) {
        if (getNativeEffect() > 0) {
            return nGenericEffectSetAttributeColor(mNativeEffect, name, value);
        }
        return false;
    }


}
