package com.shixing.sxedit;

import com.shixing.sxedit.delegate.*;
import com.shixing.sxedit.types.SXGenericEffectType;
import com.shixing.sxedit.util.TextUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 可渲染轨道的基类
 * 
 * */

public class SXRenderTrack extends SXTrack implements
        SXFilterDelegate, SXAnimationDelegate, SXGenericEffectDelegate, SXTrackAnimationDelegate, SXVideoEffectDelegate {

    SXRenderTrack(long track, long manager, SXTrackOwnType type) {
        super(track, manager, type);
    }


    /**
     * 获取变换矩阵的位置值
     *
     * @return float[]
     */
    public float[] getPosition() {
        if (mNativeManager > 0) {
            return nPosition(mNativeManager, mOwnType.value,mTrackID);
        }
        return null;
    }

    /**
     * 设置变换矩阵的位置值
     *
     * @param point 换矩阵的位置值
     */
    public void setPosition(float[] point) {
        if (mNativeManager > 0) {
            nSetPosition(mNativeManager, mOwnType.value,mTrackID, point);
        }
    }

    /**
     * 设置轨道的不透明度，范围为[0, 1]，0表示全透明，1表示不透明
     *
     * @param opacity 不透明度
     */
    public void setOpacity(float opacity) {
        if (mNativeManager > 0) {
            nSetOpacity(mNativeManager, mOwnType.value,mTrackID, opacity);
        }
    }

    /**
     * 获取不透明度
     *
     * @return 不透明度
     */
    public float getOpacity() {
        if (mNativeManager > 0) {
            return nOpacity(mNativeManager, mOwnType.value,mTrackID);
        }
        return 1;
    }

    /**
     * 设置变换矩阵的旋转值
     *
     * @param rotation 变换矩阵的旋转值
     */
    public void setRotation(float rotation) {
        if (mNativeManager > 0) {
            nSetRotation(mNativeManager,mOwnType.value, mTrackID, rotation);
        }
    }

    /**
     * 获取变换矩阵的旋转值
     *
     * @return 变换矩阵的旋转值
     */
    public float getRotation() {
        if (mNativeManager > 0) {
            return nRotation(mNativeManager,mOwnType.value, mTrackID);
        }
        return 0;
    }

    /**
     * 设置变换矩阵的缩放值
     *
     * @param scaleX 转换矩阵的 x 缩放值
     * @param scaleY 转换矩阵的 y 缩放值
     */
    public void setScale(float scaleX, float scaleY) {
        if (mNativeManager > 0) {
            nSetScale(mNativeManager,mOwnType.value, mTrackID, scaleX, scaleY);
        }
    }

    /**
     * 获取变换矩阵的缩放值
     *
     * @return 变换矩阵的缩放值
     */
    public float[] getScale() {
        if (mNativeManager > 0) {
            return nScale(mNativeManager, mOwnType.value,mTrackID);
        }
        return null;
    }

    /**
     * 获取轨道的变换矩阵
     *
     * @return float[]
     */
    public float[] getTransform() {
        if (mNativeManager > 0) {
            return nTransform(mNativeManager, mOwnType.value,mTrackID);
        }
        return null;
    }

    /**
     * 设置轨道的变换矩阵
     *
     * @param transform 变换矩阵
     */
    public void setTransform(float[] transform) {
        if (mNativeManager > 0) {
            nSetTransform(mNativeManager, mOwnType.value,mTrackID, transform);
        }
    }

    /**
     * 轨道自适应到当前Context，会改变轨道的变换矩阵
     *
     * @param coverAll 是否填充满整个画面
     * @param margin   当coverAll为false时，track在整个画面留边边距为margin的范围里适配宽高完整展示
     */
    public void fitToComposite(boolean coverAll, int margin) {
        if (mNativeManager > 0) {
            nFitToComposite(mNativeManager,mOwnType.value, mTrackID, coverAll, margin);
        }
    }

    /**
     * 设置纵向翻转
     *
     * @param verticalFlip true时表示纵向翻转
     */
    public void setVerticalFlip(boolean verticalFlip) {
        if (mNativeManager > 0) {
            nSetVerticalFlip(mNativeManager, mOwnType.value,mTrackID, verticalFlip);
        }
    }

    /**
     * 轨道是否纵向翻转
     *
     * @return 是纵向翻转返回true，否则返回false
     */
    public boolean isVerticalFlip() {
        if (mNativeManager > 0) {
            return nVerticalFlip(mNativeManager, mOwnType.value,mTrackID);
        }
        return false;
    }

    /**
     * 设置横向翻转
     *
     * @param horizontalFlip true时表示横向翻转
     */
    public void setHorizontalFlip(boolean horizontalFlip) {
        if (mNativeManager > 0) {
            nSetHorizontalFlip(mNativeManager,mOwnType.value, mTrackID, horizontalFlip);
        }
    }

    /**
     * 轨道是否横向翻转
     *
     * @return 是横向翻转返回true，否则返回false
     */
    public boolean isHorizontalFlip() {
        if (mNativeManager > 0) {
            return nHorizontalFlip(mNativeManager,mOwnType.value, mTrackID);
        }
        return false;
    }

    @Override
    public SXFilterEffect addFilter(String filterPath, double duration, int index) {
        if (mNativeManager == 0) {
            return null;
        }
        long effect = nAddFilter(mNativeManager, mOwnType.value, mTrackID, filterPath,  duration,  index);
        if (effect == 0) {
            return null;
        }
        return new SXFilterEffect(effect, mNativeManager, mTrackID);
    }

    @Override
    public SXFilterEffect addFilterByCopy(SXFilterEffect effect, int index) {
        if (mNativeManager == 0) {
            return null;
        }
        long nEffect = nAddFilterByCopy(mNativeManager, mOwnType.value, mTrackID, effect.getNativeEffect(), index);
        if (nEffect != 0) {
            return new SXFilterEffect(nEffect, mNativeManager, mTrackID);
        }
        return null;
    }

    @Override
    public boolean removeFilter(String filterId) {
        if (mNativeManager == 0 ||  TextUtils.isEmpty(filterId)) {
            return false;
        }
        return nRemoveFilter(mNativeManager, mOwnType.value, mTrackID, filterId);
    }

    @Override
    public SXFilterEffect getFilter(String filterId) {
        if (mNativeManager == 0 ||  TextUtils.isEmpty(filterId)) {
            return null;
        }

        long effect = nGetFilter(mNativeManager, mOwnType.value, mTrackID, filterId);
        if (effect == 0) {
            return null;
        }
        return new SXFilterEffect(effect, mNativeManager, mTrackID);
    }

    @Override
    public List<SXFilterEffect> getFilters() {
        if (mNativeManager > 0) {
            long[] ids = nGetFilters(mNativeManager, mOwnType.value, mTrackID);
            if (ids != null && ids.length > 0) {
                List<SXFilterEffect> effects = new ArrayList<>();
                for (long id : ids) {
                    effects.add(new SXFilterEffect(id, mNativeManager, mTrackID));
                }
                return effects;
            }
        }
        return null;
    }

    @Override
    public int getNumFilters() {
        if (mNativeManager > 0) {
            return nNumFilters(mNativeManager, mOwnType.value, mTrackID);
        }
        return 0;
    }

    @Override
    public SXAnimationEffect addAnimation(String animationPath, double duration, int index) {
        long effect = nAddAnimation(mNativeManager, mOwnType.value, mTrackID, animationPath, duration, index);
        if (effect == 0) {
            return null;
        }
        return new SXAnimationEffect(effect, mNativeManager,  mTrackID);
    }

    @Override
    public SXAnimationEffect addAnimationByCopy(SXAnimationEffect effect, int index) {
        long nEffect = nAddAnimationByCopy(mNativeManager, mOwnType.value, mTrackID, effect.getNativeEffect(), index);
        if (nEffect != 0) {
            return new SXAnimationEffect(nEffect, mNativeManager, mTrackID);
        }
        return null;
    }

    @Override
    public boolean removeAnimation(String animationId) {
        if (mNativeManager == 0 ||  TextUtils.isEmpty(animationId)) {
            return false;
        }
        return nRemoveAnimation(mNativeManager, mOwnType.value, mTrackID, animationId);
    }

    @Override
    public SXAnimationEffect getAnimation(String animationId) {
        if (mNativeManager == 0 ||  TextUtils.isEmpty(animationId)) {
            return null;
        }

        long effect = nGetAnimation(mNativeManager, mOwnType.value, mTrackID, animationId);
        if (effect == 0) {
            return null;
        }
        return new SXAnimationEffect(effect, mNativeManager,  mTrackID);
    }

    @Override
    public List<SXAnimationEffect> getAnimations() {
        if (mNativeManager > 0) {
            long[] ids = nGetAnimations(mNativeManager, mOwnType.value, mTrackID);
            if (ids != null && ids.length > 0) {
                List<SXAnimationEffect> effects = new ArrayList<>();
                for (long id : ids) {
                    effects.add(new SXAnimationEffect(id, mNativeManager,  mTrackID));
                }
                return effects;
            }
        }
        return null;
    }

    @Override
    public int getNumAnimations() {
        if (mNativeManager > 0) {
            return nNumAnimations(mNativeManager, mOwnType.value, mTrackID);
        }
        return 0;
    }


    @Override
    public SXGenericEffect addGenericEffect(SXGenericEffectType type, double duration, int index) {
        if (mNativeManager == 0) {
            return null;
        }

        long effect = nAddGenericEffect(mNativeManager, mOwnType.value, mTrackID,  type.value, duration, index);
        if (effect == 0) {
            return null;
        }

        return new SXGenericEffect(effect, mNativeManager, mTrackID);
    }

    @Override
    public SXGenericEffect addGenericEffectByCopy(SXGenericEffect effect, int index) {
        if (mNativeManager == 0) {
            return null;
        }

        long nEffect = nAddGenericEffectByCopy(mNativeManager, mOwnType.value, mTrackID, effect.getNativeEffect(), index);
        if (nEffect != 0) {
            return new SXGenericEffect(nEffect, mNativeManager, mTrackID);
        }
        return null;
    }

    @Override
    public boolean removeGenericEffect(String effectId) {
        if (mNativeManager == 0 ||  TextUtils.isEmpty(effectId)) {
            return false;
        }
        return nRemoveGenericEffect(mNativeManager, mOwnType.value, mTrackID, effectId);
    }

    @Override
    public SXGenericEffect getGenericEffect(String effectId) {
        if (mNativeManager == 0 ||  TextUtils.isEmpty(effectId)) {
            return null;
        }

        long effect = nGetGenericEffect(mNativeManager, mOwnType.value, mTrackID, effectId);
        if (effect == 0) {
            return null;
        }
        return new SXGenericEffect(effect, mNativeManager, mTrackID);
    }

    @Override
    public List<SXGenericEffect> getGenericEffects() {
        if (mNativeManager > 0) {
            long[] ids = nGetGenericEffects(mNativeManager, mOwnType.value, mTrackID);
            if (ids != null && ids.length > 0) {
                List<SXGenericEffect> effects = new ArrayList<>();
                for (long id : ids) {
                    effects.add(new SXGenericEffect(id, mNativeManager, mTrackID));
                }
                return effects;
            }
        }
        return null;
    }

    @Override
    public int getNumGenericEffects() {
        if (mNativeManager > 0) {
            return nNumVideoEffects(mNativeManager, mOwnType.value, mTrackID);
        }
        return 0;
    }

    @Override
    public SXTrackAnimationEffect addTrackAnimation(String trackAnimationPath, double duration, int index) {
        if (mNativeManager == 0 || TextUtils.isEmpty(trackAnimationPath) ) {
            return null;
        }
        long effect = nAddTrackAnimation(mNativeManager, mOwnType.value, mTrackID, trackAnimationPath,  duration,  index);
        if (effect == 0) {
            return null;
        }
        return new SXTrackAnimationEffect(effect, mNativeManager, mTrackID);
    }

    @Override
    public SXTrackAnimationEffect addTrackAnimationByCopy(SXTrackAnimationEffect effect, int index) {
        if (mNativeManager == 0) {
            return null;
        }
        long _effect = nAddTrackAnimationByCopy(mNativeManager, mOwnType.value, mTrackID, effect.getNativeEffect(), index );
        if (_effect != 0) {
            return null;
        }
        return new SXTrackAnimationEffect(_effect, mNativeManager,  mTrackID);
    }

    @Override
    public boolean removeTrackAnimation(String trackAnimationId) {
        if (mNativeManager == 0 || TextUtils.isEmpty(trackAnimationId)) {
            return false;
        }
        return nRemoveTrackAnimation(mNativeManager, mOwnType.value, mTrackID, trackAnimationId );
    }

    @Override
    public SXTrackAnimationEffect getTrackAnimation(String effectId) {
        if (mNativeManager == 0 ||  TextUtils.isEmpty(effectId)) {
            return null;
        }

        long effect = nGetTrackAnimation(mNativeManager, mOwnType.value, mTrackID, effectId);
        if (effect == 0) {
            return null;
        }
        return new SXTrackAnimationEffect(effect, mNativeManager, mTrackID);
    }

    @Override
    public List<SXTrackAnimationEffect> getTrackAnimations() {
        if (mNativeManager > 0) {
            long[] ids = nGetTrackAnimations(mNativeManager, mOwnType.value, mTrackID);
            if (ids != null && ids.length > 0) {
                List<SXTrackAnimationEffect> effects = new ArrayList<>();
                for (long id : ids) {
                    effects.add(new SXTrackAnimationEffect(id, mNativeManager,  mTrackID));
                }
                return effects;
            }
        }
        return null;
    }

    @Override
    public int getNumTrackAnimations() {
        if (mNativeManager > 0) {
            return nNumTrackAnimations(mNativeManager, mOwnType.value, mTrackID);
        }
        return 0;
    }

    @Override
    public SXVideoEffect addVideoEffect(String videoEffectPath, double duration, int index) {
        if (mNativeManager == 0 || TextUtils.isEmpty(videoEffectPath) ) {
            return null;
        }
        long effect = nAddVideoEffect(mNativeManager, mOwnType.value, mTrackID, videoEffectPath,  duration,  index);
        if (effect == 0) {
            return null;
        }
        return new SXVideoEffect(effect, mNativeManager, mTrackID);
    }

    @Override
    public SXVideoEffect addVideoEffectByCopy(SXVideoEffect effect, int index) {
        if (mNativeManager == 0) {
            return null;
        }
        long _effect = nAddGenericEffectByCopy(mNativeManager, mOwnType.value, mTrackID, effect.getNativeEffect(), index );
        if (_effect != 0) {
            return null;
        }
        return new SXVideoEffect(_effect, mNativeManager,  mTrackID);
    }

    @Override
    public boolean removeVideoEffect(String videoEffectId) {
        if (mNativeManager == 0 || TextUtils.isEmpty(videoEffectId)) {
            return false;
        }
        return nRemoveVideoEffect(mNativeManager, mOwnType.value, mTrackID, videoEffectId );
    }

    @Override
    public SXVideoEffect getVideoEffect(String videoEffectId) {
        if (mNativeManager == 0 ||  TextUtils.isEmpty(videoEffectId)) {
            return null;
        }

        long effect = nGetVideoEffect(mNativeManager, mOwnType.value, mTrackID, videoEffectId);
        if (effect == 0) {
            return null;
        }
        return new SXVideoEffect(effect, mNativeManager, mTrackID);
    }

    @Override
    public List<SXVideoEffect> getVideoEffects() {
        if (mNativeManager > 0) {
            long[] ids = nGetVideoEffects(mNativeManager, mOwnType.value, mTrackID);
            if (ids != null && ids.length > 0) {
                List<SXVideoEffect> effects = new ArrayList<>();
                for (long id : ids) {
                    effects.add(new SXVideoEffect(id, mNativeManager,  mTrackID));
                }
                return effects;
            }
        }
        return null;
    }

    @Override
    public int getNumVideoEffects() {
        if (mNativeManager > 0) {
            return nNumVideoEffects(mNativeManager, mOwnType.value, mTrackID);
        }
        return 0;
    }
}
