package com.shixing.sxedit;

import com.shixing.sxedit.util.RectF;

/**
 * 蒙版
 *
 * */

public class SXShape {

    /**
     * 创建一个 Shape
     *
     * <p>
     *     1. 使用完成后, 调用 destroy 释放
     * </p>
     *
     * */
    public SXShape() {
        mNativeShape = nCreateShape();
    }

    /**
     * 清空所有path
     */
    public void clear() {
        nClear(mNativeShape);
    }


    /**
     * 设置下一个轮廓的起点为 (x, y)
     */
    public void moveTo(float x, float y) {
        nMoveTo(mNativeShape, x, y);
    }

    /**
     * 添加一条从上一个点到(x, y)的直线
     */
    public void lineTo(float x, float y) {
        nLineTo(mNativeShape, x, y);
    }

    /**
     * 从上一个点开始添加一条三次贝塞尔曲线
     */
    public void bezierTo(float c1x, float c1y, float c2x, float c2y, float x, float y) {
        nBezierTo(mNativeShape, c1x, c1y, c2x, c2y, x, y);
    }

    /**
     * 从上一个点开始添加一条二次贝塞尔曲线
     */
    public void quadTo(float cx, float cy, float x, float y) {
        nQuadTo(mNativeShape, cx, cy, x, y);
    }

    /**
     * 以上一个点为起点添加一条弧线
     */
    public void arcTo(float x1, float y1, float x2, float y2, float radius) {
        nArcTo(mNativeShape, x1, y1, x2, y2, radius);
    }

    /**
     * 闭合路径
     */
    public void closePath() {
        nClosePath(mNativeShape);
    }

    /**
     * 添加一个矩形
     */
    public void rect(float x, float y, float w, float h) {
        nRect(mNativeShape, x, y, w, h);
    }

    /**
     * 添加一个圆角矩形
     */
    public void roundRect(float x, float y, float w, float h, float r) {
        nRoundRect(mNativeShape, x, y, w, h, r);
    }

    /**
     * 添加一个椭圆
     */
    public void ellipse(float cx, float cy, float rx, float ry) {
        nEllipse(mNativeShape, cx, cy, rx, ry);
    }

    /**
     * 添加一个圆
     */
    public void circle(float cx, float cy, float r) {
        nCircle(mNativeShape, cx, cy, r);
    }

    /**
     * 旋转一个角度
     *
     * @param degrees 旋转角度
     */
    public void rotate(float degrees) {
        nRotate(mNativeShape, degrees);
    }

    /**
     * 应用一个位移
     */
    public void translate(float x, float y) {
        nTranslate(mNativeShape, x, y);
    }

    /**
     * 缩放
     *
     * @param x x轴缩放倍数
     * @param y y轴缩放倍数
     */
    public void scale(float x, float y) {
        nScale(mNativeShape, x, y);
    }

    /**
     * 应用一个变换矩阵
     *
     * @param transform 变换矩阵
     */
    public void transform(float[] transform) {
        nApplyTransform(mNativeShape, transform);
    }

    /**
     * 销毁创建的 Shape 对象
     * */
    public void destroy() {
        if (mNativeShape > 0) {
            nDelete(mNativeShape);
        }
    }


    long mNativeShape;
    float[] mMatrixValue = new float[9];
    float[] mBoxValue = new float[4];

    SXShape(long nativeShape) {
        mNativeShape = nativeShape;
    }

    @Override
    protected void finalize() throws Throwable {
        destroy();
    }

    //=============================================
    private static native long nCreateShape();
    private static native void nClear(long nativeShape);
    private static native void nMoveTo(long nativeShape, float x, float y);
    private static native void nLineTo(long nativeShape, float x, float y);
    private static native void nBezierTo(long nativeShape, float c1x, float c1y, float c2x, float c2y, float x, float y);
    private static native void nQuadTo(long nativeShape, float cx, float cy, float x, float y);
    private static native void nArcTo(long nativeShape, float x1, float y1, float x2, float y2, float radius);
    private static native void nClosePath(long nativeShape);
    private static native void nRect(long nativeShape, float x, float y, float w, float h);
    private static native void nRoundRect(long nativeShape, float x, float y, float w, float h, float r);
    private static native void nEllipse(long nativeShape, float cx, float cy, float rx, float ry);
    private static native void nCircle(long nativeShape, float cx, float cy, float r);
    private static native void nRotate(long nativeShape, float angle);
    private static native void nTranslate(long nativeShape, float x, float y);
    private static native void nScale(long nativeShape, float x, float y);
    private static native void nApplyTransform(long nativeShape, float[] temp);
    private static native void nDelete(long nativeShape);
}
